<?php

/*
 * Server Neighborhood
 * This Plugin show an amount of your servers in the game. Such like "zero-switch" or
 * "sak.switch". Based opon the idea and behavior of "sak.switch".
 * ----------------------------------------------------------------------------------
 * Author:	undef.de
 * Version:	0.9.9
 * Date:	2009-12-06
 * Home:	http://www.undef.de/Trackmania/Plugins/
 * System:	xaseco/1.06 and up
 * ----------------------------------------------------------------------------------
 *
 * LICENSE: This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 *
 * ----------------------------------------------------------------------------------
 *
 * Dependencies: none
 */

/* The following manialink id's are used in this plugin (the 912 part of id can be changed on trouble):
 * 91200		id for manialinks
 * 91201		id for manialink Widgets
 * 91202		id for action call chat_players() (Dependencies: chat.players.php)
 * 912100-912199	id for action open server overview (nn count of servers, max. 99 server)
 */

Aseco::registerEvent('onStartup',			'sn_onStartup');
Aseco::registerEvent('onPlayerConnect',			'sn_StoreData');
Aseco::registerEvent('onPlayerDisconnect',		'sn_StoreData');
Aseco::registerEvent('onPlayerInfoChanged',		'sn_StoreData');
Aseco::registerEvent('onNewChallenge',			'sn_StoreData');
Aseco::registerEvent('onNewChallenge',			'sn_SetActive');
Aseco::registerEvent('onEndRace',			'sn_SetInactive');
Aseco::registerEvent('onEverySecond',			'sn_CheckRefresh');
Aseco::registerEvent('onPlayerManialinkPageAnswer',	'sn_CreateOverview');

global $sn_config, $SN_VERSION;

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function sn_onStartup ($aseco) {
	global $sn_config, $SN_VERSION;

	if ( version_compare(XASECO_VERSION, 1.06, '<') ) {
		trigger_error('[plugin.server_neighborhood.php] Not supported xaseco version ('. XASECO_VERSION .')! Please update to min. version 1.06!', E_USER_ERROR);
	}

	if (!$sn_config = $aseco->xml_parser->parseXML('serverneighborhood.xml')) {
		trigger_error("[plugin.server_neighborhood.php] Could not read/parse config file serverneighborhood.xml !", E_USER_ERROR);
	}

	$sn_config = $sn_config['SERVER_NEIGHBORHOOD'];

	if (strtoupper($sn_config['SHOW_SELF'][0]) == 'TRUE') {
		$sn_config['SERVER_ACCOUNTS'][0]['SERVER_NEIGHBOR_LOGIN'] = array_merge(array($sn_config['SERVER_LOGIN'][0]), $sn_config['SERVER_ACCOUNTS'][0]['SERVER_NEIGHBOR_LOGIN']);
	}

	if (!$sn_config['WIDGET'][0]['WIDGET_POS_X'][0]) {
		$sn_config['WIDGET'][0]['WIDGET_POS_X'][0] = 0;
	}
	if (!$sn_config['WIDGET'][0]['WIDGET_POS_Y'][0]) {
		$sn_config['WIDGET'][0]['WIDGET_POS_Y'][0] = 0;
	}
	if (!$sn_config['REFRESH_INTERVAL'][0]) {
		$sn_config['REFRESH_INTERVAL'][0] = 10;
	}


	$sn_config['MANIALINK_ID'] = '912';
	$sn_config['SET_ACTIVE'] = true;
	$sn_config['REFRESH_COUNT'] = 0;

	$SN_VERSION = '0.9.9';
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function sn_SetActive ($aseco, $challenge_item) {
	global $sn_config;

	$sn_config['SET_ACTIVE'] = true;
	sn_CreateWidget($aseco);
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function sn_SetInactive ($aseco, $command) {
	global $sn_config;

	if (strtolower($sn_config['SHOW_ON_END_RACE'][0]) == 'false') {
		$sn_config['SET_ACTIVE'] = false;
		sn_CreateWidget($aseco);
	}
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function sn_CheckRefresh ($aseco, $command) {
	global $sn_config;

	if ($sn_config['REFRESH_COUNT'] >= $sn_config['REFRESH_INTERVAL'][0]) {

		// Reset RefreshCount
		$sn_config['REFRESH_COUNT'] = 1;

		sn_CreateWidget($aseco);

	}
	else {
		$sn_config['REFRESH_COUNT'] += 1;		// Count up RefreshCount
	}

}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function sn_StoreData ($aseco, $player_item) {
	global $sn_config;

	if ($sn_config['STORING_PATH'][0] == '') {
		sn_onStartup($aseco);
	}


	// Set self (serverlogin)
	$sn_config['SERVER_LOGIN'][0] = $aseco->server->serverlogin;

	$CurrentPlayerCount = 0;
	$CurrentSpectatorCount = 0;
	$CurrentPlayers = array();


	$aseco->client->resetError();
	$aseco->client->query('GetPlayerList', 300, 0);
	$players = $aseco->client->getResponse();

	if ( $aseco->client->isError() ) {
		trigger_error('[' . $aseco->client->getErrorCode() . '] ' . $aseco->client->getErrorMessage(), E_USER_WARNING);
	}
	else {
		foreach ($players as $player) {
				$CurrentPlayers[$CurrentPlayerCount] = $player['NickName'];
				$CurrentPlayerCount++;
				if ($player['IsSpectator'] == 1) {
					$CurrentSpectatorCount++;
				}
		}
		$CurrentPlayerCount = $CurrentPlayerCount - $CurrentSpectatorCount;
	}

	$aseco->client->query('GetServerOptions');
	$ServerOptions = $aseco->client->getResponse();
	$ServerName		= $ServerOptions['Name'];
	$CurrentMaxPlayers	= $ServerOptions['CurrentMaxPlayers'];
	$CurrentMaxSpectators	= $ServerOptions['CurrentMaxSpectators'];

	$aseco->client->query('GetLadderServerLimits');
	$Ladder = $aseco->client->getResponse();
	$LadderMin = substr($Ladder['LadderServerLimitMin'],0, 2);
	$LadderMax = substr($Ladder['LadderServerLimitMax'],0, 2);

	$aseco->client->query('GetCurrentChallengeInfo');
	$ChallengeInfo = $aseco->client->getResponse();


	// Create Server informations
	$info = $ServerName .LF;
	$info .= $CurrentPlayerCount .LF;
	$info .= $CurrentMaxPlayers .LF;
	$info .= $CurrentSpectatorCount .LF;
	$info .= $CurrentMaxSpectators .LF;
	$info .= $LadderMin .LF;
	$info .= $LadderMax .LF;
	$info .= $ChallengeInfo['Name'] .LF;
	$info .= implode(LF, $CurrentPlayers);


	$filename = $sn_config['STORING_PATH'][0] . $sn_config['SERVER_LOGIN'][0] .'_serverinfo.txt';

	// Allows overwriting of existing files on the remote FTP server,
	// creates a stream context resource with the defined options
	$stream_context = stream_context_create( array('ftp' => array('overwrite' => true)) );

	// Opens the file for writing and truncates it to zero length
	if ($fh = fOpen($filename, 'w', 0, $stream_context)) {
		fWrite($fh, $info);
		fClose($fh);
	}
	else {
		trigger_error('[plugin.server_neighborhood.php] Could not open file "'. $filename .'" to store the Server Information!', E_USER_WARNING);
	}
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function sn_CreateWidget ($aseco) {
	global $sn_config;

	if ($sn_config['SET_ACTIVE'] == false) {
		$xml =  '<?xml version="1.0" encoding="UTF-8"?>';
		$xml .= '<manialinks id="'. $sn_config['MANIALINK_ID'] .'00">';
		$xml .= '<manialink id="'. $sn_config['MANIALINK_ID'] .'01">';
		$xml .= '</manialink>';
		$xml .= '</manialinks>';

		$aseco->client->addCall('SendDisplayManialinkPage', array($xml, 0, false));
	}
	else {
		if ( count($sn_config['SERVER_ACCOUNTS']) > 0) {

			$line_height = 3.2;
			$headerline_height = 2.1;

			$xml =  '<?xml version="1.0" encoding="UTF-8"?>';
			$xml .= '<manialinks id="'. $sn_config['MANIALINK_ID'] .'00">';
			$xml .= '<manialink id="'. $sn_config['MANIALINK_ID'] .'01">';

			// Count how many neighbors (needed for size the frame)
			$neighbor_count = 0;
			foreach ($sn_config['SERVER_ACCOUNTS'][0]['SERVER_NEIGHBOR_LOGIN'] as $filename) {
				if ($filename != '') {
					$neighbor_count++;
				}
			}

			$xml .= '<frame posn="'. $sn_config['WIDGET'][0]['WIDGET_POS_X'][0] .' '. $sn_config['WIDGET'][0]['WIDGET_POS_Y'][0] .' 20">';
			$xml .= '<quad sizen="15.2 '. ($neighbor_count * $line_height + $headerline_height + 0.75) .'" posn="0 0 0" style="Bgs1InRace" substyle="NavButton"/>';
			$xml .= '<quad sizen="14.38 2" posn="0.4 -0.36 1" style="BgsPlayerCard" substyle="BgRacePlayerName"/>';

			if (strtoupper($sn_config['WIDGET'][0]['ICON_POSITION'][0]) == 'LEFT') {
				$xml .= '<quad sizen="2.6 2.6" posn="0.6 0 2" style="Icons128x128_1" substyle="Browse"/>';
				$xml .= '<label textsize="1" sizen="10.2 0" posn="3.2 -0.5 2" text="$EEEServer Neighborhood"/>';
			}
			else {
				$xml .= '<quad sizen="2.6 2.6" posn="12 0 2" style="Icons128x128_1" substyle="Browse"/>';
				$xml .= '<label textsize="1" sizen="10.2 0" posn="1.4 -0.5 2" text="$EEEServer Neighborhood"/>';
			}


			// Generate the Server Neighborhoods
			$neighbor_count = 0;
			$link_count = 0;
			foreach ($sn_config['SERVER_ACCOUNTS'][0]['SERVER_NEIGHBOR_LOGIN'] as $filename) {
				if ($filename != '') {
					$neighbor = array();
					$neighbor = file($sn_config['STORING_PATH'][0] . $filename .'_serverinfo.txt');

					// Remove leading LFs from Lines
					if ($neighbor[0]) {
						foreach ($neighbor as &$value) {
							$value = str_replace("\r\n", '', $value);
							$value = str_replace("\n", '', $value);
						}
						$neighbor[0] = str_ireplace('$S', '', $neighbor[0]);		// Remove $S from Servername
						$neighbor[0] = str_ireplace('"', '&quot;', $neighbor[0]);	// Replace " from Servername
					}

					// Serverframe
					$xml .= '<frame posn="0 -'. ($line_height * $neighbor_count + $headerline_height) .' 0.001">';

					if ($sn_config['SERVER_LOGIN'][0] == $filename) {
						if ( function_exists('chat_players') ) {
							$xml .= '<quad sizen="14.38 '. $line_height .'" action="'. sprintf("%d%d%02d", $sn_config['MANIALINK_ID'], 0, 2) .'" posn="0.4 -0.36 0.002" style="'. $sn_config['WIDGET'][0]['SELF_STYLE'][0] .'" substyle="'. $sn_config['WIDGET'][0]['SELF_SUBSTYLE'][0] .'"/>';
						}
						else {
							$xml .= '<quad sizen="14.38 '. $line_height .'" posn="0.4 -0.36 0.002" style="'. $sn_config['WIDGET'][0]['SELF_STYLE'][0] .'" substyle="'. $sn_config['WIDGET'][0]['SELF_SUBSTYLE'][0] .'"/>';
						}
						if (strtoupper($sn_config['WIDGET'][0]['ICON_POSITION'][0]) == 'LEFT') {
							$xml .= '<quad sizen="2.2 2.7" posn="-1.3 -0.76 0.2" style="Icons64x64_1" substyle="ArrowGreen"/>';
						}
						else {
							$xml .= '<quad sizen="1.6 1.6" posn="14.75 -1.3 0.2" style="Icons64x64_1" substyle="LvlGreen"/>';
						}
					}
					else {
						if ($neighbor[0] == '') {
							$xml .= '<quad sizen="14.38 '. $line_height .'" posn="0.4 -0.36 0.002" style="'. $sn_config['WIDGET'][0]['NEIGHBOR_STYLE'][0] .'" substyle="'. $sn_config['WIDGET'][0]['NEIGHBOR_SUBSTYLE'][0] .'"/>';

							// Server-Information could not be opened so just fill the array
							$neighbor[0] = 'Server unavailable!';
							$neighbor[1] = 0;
							$neighbor[2] = 0;
							$neighbor[3] = 0;
							$neighbor[4] = 0;
							$neighbor[5] = 0;
							$neighbor[6] = 0;
						}
						else {
							$xml .= '<quad sizen="14.38 '. $line_height .'" action="'. sprintf("%d%d%02d", $sn_config['MANIALINK_ID'], 1, $link_count) .'" posn="0.4 -0.36 0.002" style="'. $sn_config['WIDGET'][0]['NEIGHBOR_STYLE'][0] .'" substyle="'. $sn_config['WIDGET'][0]['NEIGHBOR_SUBSTYLE'][0] .'"/>';
						}
					}
					$xml .= '<label textsize="1" scale="0.9" sizen="13 0" posn="1.25 -0.55 0.003" text="'. $neighbor[0] .'"/>';

					// Detailframe for this server
					$xml .= '<frame sizen="13 2" posn="1.2 -2 0.003">';

					// Players
					$xml .= '<quad sizen="1.6 1.6" posn="0 0.2 0.004" style="Icons128x128_1" substyle="Hotseat"/>';
					$xml .= '<label sizen="3.5 0" posn="1.58 0 0.004" textsize="1" scale="0.75" text="'. (($neighbor[1] == $neighbor[2]) ? $sn_config['WIDGET'][0]['SERVER_FULL_COLOR'][0] : $sn_config['WIDGET'][0]['SERVER_NORMAL_COLOR'][0]). $neighbor[1] .'/'. $neighbor[2] .'"/>';

					// Spectators
					$xml .= '<quad sizen="1.6 1.6" posn="4.45 0.2 0.004" style="Icons128x128_1" substyle="ChallengeAuthor"/>';
					$xml .= '<label sizen="3.5 0" posn="5.9 0 0.004" textsize="1" scale="0.75" text="'. (($neighbor[3] == $neighbor[4]) ? $sn_config['WIDGET'][0]['SERVER_FULL_COLOR'][0] : $sn_config['WIDGET'][0]['SERVER_NORMAL_COLOR'][0]). $neighbor[3] .'/'. $neighbor[4] .'"/>';

					// Ladder
					$xml .= '<quad sizen="1.6 1.5" posn="9 0.3 0.004" style="Icons128x128_1" substyle="Rankings"/>';
					$xml .= '<label sizen="3.5 0" posn="10.4 0 0.004" textsize="1" scale="0.75" text="'. $sn_config['WIDGET'][0]['SERVER_NORMAL_COLOR'][0] . $neighbor[5] .'-'. $neighbor[6] .'k"/>';

					$xml .= '</frame>'; // END Detailframe for this server

					$xml .= '</frame>'; // END Serverframe
					$neighbor_count++;

					// Support for max. 100 servers
					if ($neighbor_count == 101) {
						break;
					}
				}
				// Need to count always!
				$link_count++;
			}
			$xml .= '</frame>';

			$xml .= '</manialink>';
			$xml .= '</manialinks>';

			$aseco->client->addCall('SendDisplayManialinkPage', array($xml, 0, false));
		}
		else {
			trigger_error('[plugin.server_neighborhood.php] No other server accounts given. Do not show the Widget!', E_USER_WARNING);
		}
	}
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function sn_CreateOverview ($aseco, $answer) {
	global $sn_config, $SN_VERSION;

	if ( ($answer[2] >= sprintf("%d%d%02d", $sn_config['MANIALINK_ID'], 1, 0) ) && ($answer[2] <= sprintf("%d%d%02d", $sn_config['MANIALINK_ID'], 1, 99)) ) {

		$player = $aseco->server->players->getPlayer($answer[1]);
		$neighborid = intval( str_replace($sn_config['MANIALINK_ID'].'1', '', $answer[2]) );

		if ( file_exists($sn_config['STORING_PATH'][0] . $sn_config['SERVER_ACCOUNTS'][0]['SERVER_NEIGHBOR_LOGIN'][$neighborid] .'_serverinfo.txt') ) {
			$neighbor = file($sn_config['STORING_PATH'][0] . $sn_config['SERVER_ACCOUNTS'][0]['SERVER_NEIGHBOR_LOGIN'][$neighborid] .'_serverinfo.txt');

			// Remove leading LFs
			foreach ($neighbor as &$value) {
				$value = str_replace(LF, '', $value);
			}
			$neighbor[0] = str_ireplace('$S', '', $neighbor[0]);	// Remove $S from Servername

			$head = 'Players on '. $neighbor[0];
			$msg = array();
			$msg[] = array('','$l[http://www.undef.de/Trackmania/Plugins/]Server Neighborhood/'. $SN_VERSION .'$l');



			// Check which LinkMode we can use. If ServerMaxPlayer is reached,
			// try to 'spectate'. If ServerMaxSpectator is reached, do not show an link.
			if ($neighbor[1] < $neighbor[2]) {
				$TmtpLink = '$l[tmtp://#join='. $sn_config['SERVER_ACCOUNTS'][0]['SERVER_NEIGHBOR_LOGIN'][$neighborid] .']$F00CLICK HERE TO JOIN$Z$L';
			}
			else if ($neighbor[3] < $neighbor[4]) {
				$TmtpLink = '$l[tmtp://#spectate='. $sn_config['SERVER_ACCOUNTS'][0]['SERVER_NEIGHBOR_LOGIN'][$neighborid] .']$F00CLICK HERE TO SPECTATE$Z$L';
			}
			else {
				$TmtpLink = '$F00SORRY, SERVER FULL, NO SWITCH POSSIBLE!$Z';
			}
			$msg[] = array('','Current Track: '. $neighbor[7] .'$Z - '. $TmtpLink);


			$player->msgs = array();
			$player->msgs[0] = array(1, $head, array(1.3, 0.1, 1.2), array('Icons128x128_1', 'Buddies'));


			// Create list of players
			$player_count = 1;
			$lines = 0;
			$offset = 8;
			while ($neighbor[$offset] != '') {

				// Format nickname & login
				$ply = '$S{#black}'. $neighbor[$offset] .'$Z';

				$msg[] = array(sprintf("%03d", $player_count).'.', $ply);
				$player_count++;
				$lines++;
				$offset++;
				if ($lines > 14) {
						$player->msgs[] = $msg;
						$lines = 0;
						$msg = array();
						$msg[] = array('','Current Track: '. $neighbor[7] .'$Z - '. $TmtpLink);
				}
			}

			// Add if last batch exists
			if (count($msg) > 1) {
				$player->msgs[] = $msg;
			}

			if (count($player->msgs) >= 1) {
				// display ManiaLink message
				display_manialink_multi($player);
			}
			else {  // == 1
				$aseco->client->query('ChatSendServerMessageToLogin', $aseco->formatColors('{#server}> {#error}No Serverstatus found!'), $player->login);
			}
		}
		else {
			trigger_error('[plugin.server_neighborhood.php] Could not find "'. $sn_config['STORING_PATH'][0] . $sn_config['SERVER_ACCOUNTS'][0]['SERVER_NEIGHBOR_LOGIN'][$neighborid] .'_serverinfo.txt", NeighborId "'. $neighborid .'"', E_USER_WARNING);
		}
	}
	elseif ($answer[2] == sprintf("%d%d%02d", $sn_config['MANIALINK_ID'], 0, 2) ) {
		if (function_exists('chat_players')) {
			$command['author'] = $aseco->server->players->getPlayer($answer[1]);
			$command['params'] = ' ';
			chat_players($aseco, $command);
		}
	}
}

?>
